@@ -1,4 +1,5 @@
 import logging.config
+import socket
 import warnings
 from itertools import chain
 from typing import List, Union
@@ -6,7 +7,9 @@ from typing import List, Union
 import overpy
 from tqdm.auto import tqdm
 
-from pyridy.osm.utils import QueryResult, OSMRailwayLine
+from pyridy.osm.utils import QueryResult, OSMLevelCrossing, OSMRailwaySwitch, OSMRailwaySignal, OSMRailwayLine, \
+    OSMRailwayElement
+from pyridy.utils.tools import internet, requires_internet
 
 logger = logging.getLogger(__name__)
 
@@ -51,7 +54,9 @@ class OSMRegion:
 
         self.desired_railway_types = desired_railway_types
         self.ways: List[overpy.Way, overpy.RelationWay] = []
+
         self.railway_lines: List[OSMRailwayLine] = []
+        self.railway_elements: List[OSMRailwayElement] = []
 
         self.query_results = {rw_type: {"track_query": QueryResult,
                                         "route_query": QueryResult} for rw_type in self.desired_railway_types}
@@ -66,7 +71,7 @@ class OSMRegion:
 
     def _create_query(self, railway_type: str):
         if railway_type not in OSMRegion.supported_railway_types:
-            raise ValueError("Your desired railway type %s is not supported" % railway_type)
+            raise ValueError("The desired railway type %s is not supported" % railway_type)
 
         track_query = """(node[""" + "railway" + """=""" + railway_type + """](""" + str(self.lat_sw) + """,""" + str(
             self.lon_sw) + """,""" + str(self.lat_ne) + """,""" + str(self.lon_ne) + """);
@@ -98,21 +103,37 @@ class OSMRegion:
             railway_types = self.desired_railway_types
 
         # Download data for all desired railway types
-        for railway_type in tqdm(railway_types):
-            trk_query, rou_query = self._create_query(railway_type=railway_type)
-            trk_result = QueryResult(self.query_overpass(trk_query), railway_type)
-            rou_result = QueryResult(self.query_overpass(rou_query), railway_type)
-            self.query_results[railway_type]["track_query"] = trk_result
-            self.query_results[railway_type]["route_query"] = rou_result
+        if internet():
+            for railway_type in tqdm(railway_types):
+                # Create Overpass queries and try downloading them
+                trk_query, rou_query = self._create_query(railway_type=railway_type)
+                trk_result = QueryResult(self.query_overpass(trk_query), railway_type)
+                rou_result = QueryResult(self.query_overpass(rou_query), railway_type)
+                self.query_results[railway_type]["track_query"] = trk_result
+                self.query_results[railway_type]["route_query"] = rou_result
 
-            for rel in rou_result.result.relations:
-                rel_way_ids = [mem.ref for mem in rel.members if type(mem) == overpy.RelationWay and not mem.role]
-                rel_ways = [w for w in rou_result.result.ways if w.id in rel_way_ids]
+                # Convert relation result to OSMRailwayLine objects
+                for rel in rou_result.result.relations:
+                    rel_way_ids = [mem.ref for mem in rel.members if type(mem) == overpy.RelationWay and not mem.role]
+                    rel_ways = [w for w in rou_result.result.ways if w.id in rel_way_ids]
 
-                sort_order = {id: idx for id, idx in zip(rel_way_ids, range(len(rel_way_ids)))}
-                rel_ways.sort(key=lambda w: sort_order[w.id])
+                    sort_order = {id: idx for id, idx in zip(rel_way_ids, range(len(rel_way_ids)))}
+                    rel_ways.sort(key=lambda w: sort_order[w.id])
 
-                self.railway_lines.append(OSMRailwayLine(rel.id, rel_ways, rel.tags, rel.members))
+                    self.railway_lines.append(OSMRailwayLine(rel.id, rel_ways, rel.tags, rel.members))
+
+                for n in trk_result.result.nodes:
+                    if "railway" in n.tags:
+                        if n.tags["railway"] == "level_crossing":
+                            self.railway_elements.append(OSMLevelCrossing(n))
+                        elif n.tags["railway"] == "signal":
+                            self.railway_elements.append(OSMRailwaySignal(n))
+                        elif n.tags["railway"] == "switch":
+                            self.railway_elements.append(OSMRailwaySwitch(n))
+                        else:
+                            pass
+        else:
+            logger.warning("Cant download OSM data because of not internet connection")
 
     def query_overpass(self, query: str, attempts: int = 3):
         for a in range(attempts):
@@ -127,6 +148,8 @@ class OSMRegion:
                 logger.warning("OverpassTooManyRequest, retrying".format(e))
             except overpy.exception.OverpassBadRequest as e:
                 logger.warning("OverpassTooManyRequest, retrying".format(e))
+            except socket.timeout as e:
+                logger.warning("Socket timeout, retrying".format(e))
         else:
             raise RuntimeError("Could download OSM data via Overpass after %d attempts." % attempts)
         return result
@@ -149,6 +172,9 @@ class OSMRegion:
 
         return ways
 
+    def get_switches(self) -> list[OSMRailwayElement]:
+        return [el for el in self.railway_elements if type(el) == OSMRailwaySwitch]
+
     @property
     def lon_sw(self):
         return self._lon_sw
